@@ -3,7 +3,7 @@
  * Released under version 2 of the Gnu Public License.
  * By Chris Brady, cbrady@sgi.com
  * ----------------------------------------------------
- * MemTest86+ V1.15 Specific code (GPL V2.0)
+ * MemTest86+ V1.30 Specific code (GPL V2.0)
  * By Samuel DEMEULEMEESTER, sdemeule@memtest.org
  * http://www.x86-secret.com - http://www.memtest.org
  */
@@ -17,6 +17,7 @@
 
 extern short memsz_mode;
 extern short firmware;
+int beepmode;
 
 struct cpu_ident cpu_id;
 ulong st_low, st_high;
@@ -51,7 +52,7 @@ static void display_init(void)
  	for(i=0, pp=(char *)(SCREEN_ADR+1); i<TITLE_WIDTH; i++, pp+=2) {
   	*pp = 0x20;
  	}
- 	cprint(0, 0, "      Memtest86  v1.15      ");
+ 	cprint(0, 0, "      Memtest86  v1.30      ");
 
  	for(i=0, pp=(char *)(SCREEN_ADR+1); i<2; i++, pp+=30) {
 	*pp = 0xA4;
@@ -96,6 +97,9 @@ void init(void)
 
 	/* setup pci */
 	pci_init();
+	
+	/* setup beep mode */
+	beepmode = BEEP_MODE;
 
 	v->test = 0;
 	v->testsel = -1;
@@ -124,7 +128,7 @@ void init(void)
 	cprint(4, COL_MID,"Pattern: ");
 	cprint(LINE_INFO-2, 0, " WallTime   Cached  RsvdMem   MemMap   Cache  ECC  Test  Pass  Errors ECC Errs");
 	cprint(LINE_INFO-1, 0, " ---------  ------  -------  --------  -----  ---  ----  ----  ------ --------");
-	cprint(LINE_INFO, COL_TST, "Std");
+	cprint(LINE_INFO, COL_TST, " Std");
 	cprint(LINE_INFO, COL_PASS, "    0");
 	cprint(LINE_INFO, COL_ERR, "     0");
 	cprint(LINE_INFO+1, 0, " -----------------------------------------------------------------------------");
@@ -383,6 +387,14 @@ void cpu_type(void)
 				l2_cache = (cpu_id.cache_info[11] << 8);
 				l2_cache += cpu_id.cache_info[10];
 				break;
+			case 13: 
+				cprint(LINE_CPU, 0, "AMD K6-III+"); 
+				off = 11; 
+				l1_cache = cpu_id.cache_info[3]; 
+				l1_cache += cpu_id.cache_info[7]; 
+				l2_cache = (cpu_id.cache_info[11] << 8); 
+				l2_cache += cpu_id.cache_info[10]; 
+				break;
 			}
 			break;
 		case 6:
@@ -440,29 +452,34 @@ void cpu_type(void)
 			l1_cache += cpu_id.cache_info[7];
 			break;
 		case 15:
+			l1_cache = cpu_id.cache_info[3];
+			l1_cache += cpu_id.cache_info[7];
+      l2_cache = (cpu_id.cache_info[11] << 8);
+      l2_cache += cpu_id.cache_info[10];
 			switch(cpu_id.model) {
-			case 4:
-			case 7:
-			case 8:
-			case 11:
-			case 12:
-			case 15:
 			default:
 				cprint(LINE_CPU, 0, "AMD Athlon 64");
-                                off = 13;
-                                l1_cache = cpu_id.cache_info[3];
-				l1_cache += cpu_id.cache_info[7];
-                                l2_cache = (cpu_id.cache_info[11] << 8);
-                                l2_cache += cpu_id.cache_info[10];
-                                break;
+             off = 13;
+             break;
 			case 5:
-				cprint(LINE_CPU, 0, "AMD Opteron");
-                                off = 11;
-                                l1_cache = cpu_id.cache_info[3];
-				l1_cache += cpu_id.cache_info[7];
-                                l2_cache = (cpu_id.cache_info[11] << 8);
-                                l2_cache += cpu_id.cache_info[10];
-                                break;
+				cprint(LINE_CPU, 0, "AMD Opteron (0.13)");
+             off = 18;
+             break;
+      case  4:
+      case 12:			
+			if (l2_cache == 256) {
+				cprint(LINE_CPU, 0, "AMD Sempron (0.13)");
+             off = 18;                        	
+      } else {
+				cprint(LINE_CPU, 0, "Athlon 64 (0.13)");
+             off = 16; 
+      }  	
+    				 break;
+   		case 15:
+				cprint(LINE_CPU, 0, "Athlon 64 (0.09)");
+             off = 16;
+             break;          
+             
 			}
 			break;
 		}
@@ -654,10 +671,16 @@ void cpu_type(void)
 			switch(cpu_id.model) {
 			case 0:
 			case 1:			
-				if (l2_cache == 128) {
+                                if (l2_cache == 128) {
                                 cprint(LINE_CPU, 0, "Celeron (0.18)");
                                 off = 14;	
-                                } else {
+                                } else if (cpu_id.pwrcap == 0x0B) {
+                                cprint(LINE_CPU, 0, "Xeon DP (0.18)");
+                                off = 14;
+                                } else if (cpu_id.pwrcap == 0x0C) {
+                                cprint(LINE_CPU, 0, "Xeon MP (0.18)");
+                                off = 14;                                	
+                        	} else {
                                 cprint(LINE_CPU, 0, "Pentium 4 (0.18)");
                                 off = 16;
                         	}
@@ -666,16 +689,29 @@ void cpu_type(void)
                                 if (l2_cache == 128) {
                                 cprint(LINE_CPU, 0, "Celeron (0.13)");
                                 off = 14;	
-                                } else {
+                                } else if (cpu_id.pwrcap == 0x0B) {
+                                cprint(LINE_CPU, 0, "Xeon DP (0.13)");
+                                off = 14;
+                                } else if (cpu_id.pwrcap == 0x0C) {
+                                cprint(LINE_CPU, 0, "Xeon MP (0.13)");
+                                off = 14;                                	
+                        	} else {
                                 cprint(LINE_CPU, 0, "Pentium 4 (0.13)");
                                 off = 16;
                         	}
                                 break;
 			case 3:
+			case 4:
                                 if (l2_cache == 256) {
                                 cprint(LINE_CPU, 0, "Celeron (0.09)");
                                 off = 14;	
-                                } else {
+                                } else if (cpu_id.pwrcap == 0x0B) {
+                                cprint(LINE_CPU, 0, "Xeon DP (0.09)");
+                                off = 14;
+                                } else if (cpu_id.pwrcap == 0x0C) {
+                                cprint(LINE_CPU, 0, "Xeon MP (0.09)");
+                                off = 14;                                	
+                        	} else {
                                 cprint(LINE_CPU, 0, "Pentium 4 (0.09)");
                                 off = 16;
                         	}
@@ -686,20 +722,50 @@ void cpu_type(void)
 		}
 		break;
 
-	/* Cyrix Processors with CPUID */
+	/* VIA/Cyrix Processors with CPUID */
 	case 'C':
-		switch(cpu_id.model) {
-		case 0:
-			cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
-			off = 16;
-			break;
-		case 4:
-			cprint(LINE_CPU, 0, "Cyrix GXm");
-			off = 9;
-			break;
+		switch(cpu_id.type) {
+		case 5:
+			switch(cpu_id.model) {
+			case 0:
+				cprint(LINE_CPU, 0, "Cyrix 6x86MX/MII");
+				off = 16;
+				break;
+			case 4:
+				cprint(LINE_CPU, 0, "Cyrix GXm");
+				off = 9;
+				break;
 		}
 		return;
 		break;
+		case 6: // VIA C3
+			switch(cpu_id.model) {
+			case 6:
+				cprint(LINE_CPU, 0, "Cyrix III");
+				off = 9;
+				break;
+			case 7:
+				if (cpu_id.step < 8) {
+				cprint(LINE_CPU, 0, "VIA C3 Samuel2");	off = 14;
+				} else {
+				cprint(LINE_CPU, 0, "VIA C3 Ezra-T");  off = 13;				
+				}	
+				break;
+			case 8:
+				cprint(LINE_CPU, 0, "VIA C3 Ezra-T");
+				off = 13;
+				break;
+			case 9:
+				cprint(LINE_CPU, 0, "VIA C3 Nehemiah");
+				off = 15;
+				break;
+			}
+			// L1 = L2 = 64 KB from Cyrix III to Nehemiah
+			l1_cache = 64;
+			l2_cache = 64;
+			break;
+		}
+		break;
 
 	/* Unknown processor */
 	default:
@@ -778,7 +844,7 @@ void cpu_type(void)
 		dprint(LINE_CPU+3, 15, speed, 6, 0);
 	}
 
-	/* Record the sarting time */
+	/* Record the starting time */
         asm __volatile__ ("rdtsc":"=a" (v->startl),"=d" (v->starth));
         v->snapl = v->startl;
         v->snaph = v->starth;
